# Exhibition d'une fonction de navigation en faisant une descente de gradient sur le potentiel reconstruit avec la fonction de Green

import math
from green import *

import matplotlib.pyplot as plt


# Geometrie du cas test
centres = [0, 2j, -2j] # Le premier cercle est ext\'erieur
rayons = [10., 0.1, 1.]
if (len(centres) != len(rayons)):
    print "Verifier nombre de cercles"
for cen, ray in zip(centres[1:], rayons[1:]):
    if (abs(cen-centres[0])+ray > rayons[0]):
        print "Verifier que le premier cercle est exterieur aux autres"

def cl_dirichlet(t, j): # Valeur prise par la fonction harmonique au bord
    if j==1:
        res = 0
    else:
        res = 1
    return res

# Parametres de construction de la fonction de Green
N = 5 # Termes du developpement
npts = 15 # Points de collocation par cercle

# Parametres de la descente de gradient
xn = -7j # point de depart de la descente
if (point_exterieur(xn, centres, rayons)):
    print "Verifier le point de d\'epart"
alpha = 1e-2 # descente \`a pas fixe
eps = 1e-1 # condition d'arret de la descente
pas = 1e-4 # pas pour le calcul de la d\'eriv\'ee


# Descente de gradient
trajectoire = [xn]
while ( abs(xn-centres[1]) > rayons[1]+eps ):
    # evaluation du gradient de la fonction harmonique
    du_dx, du_dy = eval_grad_u(xn, cl_dirichlet, centres, rayons, npts, N, pas)
    
    # choix du prochain point
    norme_gradient = math.sqrt(du_dx*du_dx + du_dy*du_dy)
    xn = xn - alpha*(du_dx + 1j*du_dy)/norme_gradient
    trajectoire.append(xn)

    
# Affichage
xmin = -rayons[0]
xmax = rayons[0]
xstep = abs(xmax-xmin)/20.
xvals = np.arange(xmin, xmax, xstep)
xpos = [x+1j*y for x in xvals for y in xvals
        if point_interieur(x+1j*y, centres, rayons)]

zres = []
for point in xpos:
    tmp = eval_u(point, cl_dirichlet, centres, rayons, npts, N)
    zres.append(tmp)

xx = [e.real for e in xpos]
yy = [e.imag for e in xpos]
sc = plt.scatter(xx, yy, c=zres, edgecolors='none')
plt.colorbar(sc)

tx = [e.real for e in trajectoire]
ty = [e.imag for e in trajectoire]
plt.scatter(tx, ty, c='black')

plt.gca().set_aspect('equal', adjustable='box')
plt.title("Descente de gradient sur la solution harmonique")
plt.show()
