# Error on Hadamard's variation of the Green's function for the Laplacian

import math
from green import *

def eqm(valeurs):
    tmp = sum([e*e for e in valeurs])
    return math.sqrt(tmp)/len(valeurs)

# Config
marge = 1.
perturbList = [0.1,
               0.09, 0.08, 0.07, 0.06, 0.05, 0.04, 0.03, 0.02, 0.01,
               9e-3, 8e-3, 7e-3, 6e-3, 5e-3,
               1e-3, 5e-4, 1e-4]
N=10
npts=32
centres = [0., 2./5.] # Outer circle is first in list
rayons = [100., 5.]
if (len(centres) != len(rayons)):
    print "Verifier nombre de cercles"
for cen, ray in zip(centres[1:], rayons[1:]):
    if (abs(cen-centres[0])+ray > rayons[0]):
        print "Verifier que le premier cercle est exterieur aux autres"

perturbList = [e*rayons[1] for e in perturbList]

print N, npts, marge

# Perturbation
centres_pert = list(centres)
rayons_pert = list(rayons)
i_pert = -1

# Grid of points
xmin = -rayons[0]
xmax = rayons[0]
xstep = rayons[0]/10.
xvals = np.arange(xmin, xmax, xstep)

for perturb in perturbList:
    rayons_pert[i_pert] = rayons[i_pert] + perturb
    cen = centres[i_pert]
    ray = rayons[i_pert]
    
    def delta_rho(t):
        return perturb
    
    xpos = [x+1j*y for x in xvals for y in xvals
            if point_interieur(x+1j*y, centres_pert, rayons, marge)]
    
    zres = []
    res = []
    for eta in xpos:
        # G0(eta, .) on initial domain
        centres_T, rayons_T = inversion_domaine(centres, rayons, eta)
        z_T = points_collocation(centres_T, rayons_T, npts)
        a,b,d,e = coeffs_harmonique(centres_T, N, z_T)
        
        g0_eta = fonction_green(centres, rayons, eta, centres_T, a,b,d,e)
        grad_g0_eta = gradient_green(centres, eta, centres_T, a,b,d)

        # G1(eta, .) on perturbed domain
        centres_pert_T, rayons_pert_T = inversion_domaine(centres_pert, rayons_pert, eta)
        z_T = points_collocation(centres_pert_T, rayons_pert_T, npts)
        a,b,d,e = coeffs_harmonique(centres_pert_T, N, z_T)
        
        g1_eta = fonction_green(centres_pert, rayons_pert, eta, centres_pert_T, a,b,d,e)
        
        for zeta in xpos:
            # G0(zeta, .) on initial domain
            centres_T, rayons_T = inversion_domaine(centres, rayons, zeta)
            z_T = points_collocation(centres_T, rayons_T, npts)
            a,b,d,e = coeffs_harmonique(centres_T, N, z_T)
            
            g0_zeta = fonction_green(centres, rayons, zeta, centres_T, a,b,d,e)
        
            # direct computation of G0(eta, zeta) on initial domain
            g0 = g0_eta(zeta)
            
            # direct computation of G1(eta, zeta) on perturbed domain
            g1 = g1_eta(zeta)
            
            # Hadamard variation : delta_G 
            d1 = delta_green_n(cen, ray, delta_rho, g0_eta, g0_zeta)
            
            # actual variation : delta G * = G* - G
            d2 = g1 - g0
            
            # epsilon_g
            tmp = abs(d2 - d1) / (abs(d2) + abs(d1))
            if not math.isnan(tmp): # g is not defined outside of domaine.
                print eta, zeta
                zres.append(tmp)
                res.append(abs(d2-d1)/abs(perturb))
    
    print perturb, perturb/rayons[1], eqm(zres), sum(zres)/len(zres), eqm(res), sum(res)/len(res)
