# Demonstrate trajectory generation from a navigation function

import math
from green import *

import matplotlib.pyplot as plt
import numpy as np


# Test case geometry
centres = [0, 2j,
           -20-20j, 20-40j, 40+40j, 20, 40-20j,
           -60+40j, -5-40j, -40+60j, 75+25j, 55+10j,
           10+20j, -15+25j, -55+5j, -40-40j, -30+10j] # [outer, destination, obstacle 1, ...]
rayons = [100., 1,
          5., 5., 5., 5., 5.,
          5., 5., 5., 5., 5.,
          5., 5., 5., 5., 5]
if (len(centres) != len(rayons)):
    print "Verifier nombre de cercles"
for cen, ray in zip(centres[1:], rayons[1:]):
    if (abs(cen-centres[0])+ray > rayons[0]):
        print "Verifier que le premier cercle est exterieur aux autres"

def cl_dirichlet(t, j): # Value on edge of domain
    if j==1:
        res = 0
    else:
        res = 1
    return res

# Parameters for building the Green's function
N = 10 # Laurent expansion
npts = 30 # Collocation points

# Parameters of gradient descent
departs = [40-40j, 35-55j, 50-25j, -80-40j, -80+50j] # starting point
alpha = 5e-1 # fixed step descent
eps = 1e-1 # end condition
pas = 1e-3 # step for computing the derivative


# Gradient descent
def descente(xn):
    if (point_exterieur(xn, centres, rayons)):
        print "Verifier le point de d\'epart"
    trajectoire = [xn]
    while ( abs(xn-centres[1]) > rayons[1]+eps ):
        # evaluate the gradient of the harmonic function
        du_dx, du_dy = eval_grad_u(xn, cl_dirichlet, centres, rayons, npts, N, pas)
        
        # choose next point
        norme_gradient = math.sqrt(du_dx*du_dx + du_dy*du_dy)
        xn = xn - alpha*(du_dx + 1j*du_dy)/norme_gradient
        trajectoire.append(xn)
    return trajectoire

    
# background
xmin = -rayons[0]
xmax = rayons[0]
xstep = 1.
xvals = np.arange(xmin, xmax, xstep)
xpos = [x+1j*y for x in xvals for y in xvals
        if point_interieur(x+1j*y, centres, rayons)]

zres = []
for point in xpos:
    tmp = eval_u(point, cl_dirichlet, centres, rayons, npts, N)
    zres.append(tmp)

xx = [e.real for e in xpos]
yy = [e.imag for e in xpos]
sc = plt.scatter(xx, yy, c=zres, edgecolors='none')
plt.colorbar(sc)

# trajectories
trajectoires = []
for xn in departs:
    plt.scatter(xn.real, xn.imag, s=100)
    trajectoire = descente(xn)
    tx = [e.real for e in trajectoire]
    ty = [e.imag for e in trajectoire]
    plt.scatter(tx, ty, c='black', s=10)

# plot results
plt.gca().set_aspect('equal', adjustable='box')
plt.title("Descente de gradient sur la solution harmonique")
plt.show()
