# Variation des trajectoires pour une perturbation de l'obstacle

import math
from green import *
import matplotlib.pyplot as plt

# Geometrie du cas test
centres = [0., 5j+1, -0.2-2j+1] # exterieur, destination, obstacle
rayons = [10., 0.1, 1.]
if (len(centres) != len(rayons)):
    print "Verifier nombre de cercles"
for cen, ray in zip(centres[1:], rayons[1:]):
    if (abs(cen-centres[0])+ray > rayons[0]):
        print "Verifier que le premier cercle est exterieur aux autres"

def cl_dirichlet(t, j): # Valeur prise par la fonction harmonique au bord
    if j==1:
        res = 0
    else:
        res = 1
    return res

# Parametres de construction de la fonction de Green
N = 5 # Termes du developpement
npts = 15 # Points de collocation par cercle

# Parametres de la descente de gradient
xn = -8j+1 # point de depart de la descente
if (point_exterieur(xn, centres, rayons)):
    print "Verifier le point de d\'epart"
alpha = 1e-1 # descente \`a pas fixe
eps = 1e-1 # condition d'arret de la descente
pas = 1e-4 # pas pour le calcul de la d\'eriv\'ee

print "% N, npts, xn, alpha, eps, pas"
print "%", N, npts, xn, alpha, eps, pas

# Descentes de gradient
def choix(xn, cl_dirichlet, cens, rays, npts, N, pas):
    # evaluation du gradient de la fonction harmonique
    du_dx, du_dy = eval_grad_u(xn, cl_dirichlet, centres, rayons, npts, N, pas)
    
    # choix du prochain point
    norme_gradient = math.sqrt(du_dx*du_dx + du_dy*du_dy)
    try:
        xn = xn - alpha*(du_dx + 1j*du_dy)/norme_gradient
    except ZeroDivisionError:
        pass
    return xn

trajectoire = [xn]
while ( abs(xn-centres[1]) > rayons[1]+eps ):
    xn = choix(xn, cl_dirichlet, centres, rayons, npts, N, pas)
    trajectoire.append(xn)

# Affichage
print "%", centres
print "%", rayons
for e in trajectoire:
    print e.real, e.imag


ax = plt.gca()

## trace du fond de carte
for cen, ray in zip(centres, rayons):
    circle = plt.Circle((cen.real, cen.imag), ray, fill=None)
    ax.add_artist(circle)

## trace des trajectoires
tx = [e.real for e in trajectoire]
ty = [e.imag for e in trajectoire]
plt.scatter(tx, ty, c='black')

## montrer la figure
plt.gca().set_aspect('equal', adjustable='box')
plt.title("Trajectoires pour deux positions d'obstacles")
plt.show()

